@@ -318,7 +318,9 @@ pid_t fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 
 			if ((slash = strrchr(session->file_on_disk, '/')) != NULL) {
 				*slash = '\0';
-				chdir(session->file_on_disk);
+				if (chdir(session->file_on_disk) == -1) {
+					log_file_error(session, session->file_on_disk, "couldn't change to directory");
+				}
 				*slash = '/';
 			}
 
