@@ -67,11 +67,10 @@
 #define rs_NONE                  0
 #define rs_QUIT_SERVER           1
 #define rs_UNBAN_CLIENTS         2
-#define rs_TOUCH_LOGFILES        3
+#define rs_UNLOCK_LOGFILES       3
+#define rs_CLEAR_CACHE           4
 
-#ifdef HAVE_SSL
 #define RANDOM_BUFFER_SIZE     512
-#endif
 
 typedef struct {
 	char *config_dir;
@@ -1244,13 +1243,19 @@ void TERM_handler(int sig) {
 	received_signal = rs_QUIT_SERVER;
 }
 
+void HUP_handler(int sig) {
+	received_signal = rs_UNLOCK_LOGFILES;
+}
+
 void USR1_handler(int sig) {
 	received_signal = rs_UNBAN_CLIENTS;
 }
 
+#ifdef HAVE_CACHE
 void USR2_handler(int sig) {
-	received_signal = rs_TOUCH_LOGFILES;
+	received_signal = rs_CLEAR_CACHE;
 }
+#endif
 
 /* Fill a filedescriptor set with sockets.
  */
@@ -1497,9 +1502,7 @@ int run_server(t_settings *settings) {
 	int                admin_socket;
 #endif
 #ifdef HAVE_SSL
-#ifndef HAVE_DEV_URANDOM
 	char               random_buffer[RANDOM_BUFFER_SIZE];
-#endif
 #endif
 	pid_t              pid;
 	t_binding          *binding;
@@ -1508,6 +1511,7 @@ int run_server(t_settings *settings) {
 	struct timeval     select_timeout;
 	struct stat        status;
 	mode_t             access_rights;
+	t_ip_addr          ip_addr;
 
 	config = default_config();
 	if (chdir(settings->config_dir) == -1) {
@@ -1548,12 +1552,12 @@ int run_server(t_settings *settings) {
 	}
 
 #ifdef HAVE_SSL
-#ifdef HAVE_DEV_URANDOM
-	ssl_init(NULL, 0);
-#else
 	fill_random_buffer(config, random_buffer, RANDOM_BUFFER_SIZE);
-	ssl_init(random_buffer, RANDOM_BUFFER_SIZE);
-#endif
+	if (ssl_init(random_buffer, RANDOM_BUFFER_SIZE) == -1) {
+		perror("ssl initialize");
+		return EXIT_FAILURE;
+	}
+
 	binding = config->binding;
 	while (binding != NULL) {
 		if (binding->use_ssl) {
@@ -1645,8 +1649,13 @@ int run_server(t_settings *settings) {
 			fprintf(stderr, "\nError while changing root to %s!\n", config->server_root);
 			return EXIT_FAILURE;
 		} while (false);
-	} else {
-		chdir("/");
+#ifdef CYGWIN
+	} else if (chdir("/cygdrive/c") == -1) {
+#else
+	} else if (chdir("/") == -1) {
+#endif
+		fprintf(stderr, "\nError while changing to root directory!\n");
+		return EXIT_FAILURE;
 	}
 
 	/* Change userid
@@ -1671,8 +1680,11 @@ int run_server(t_settings *settings) {
 	}
 
 	signal(SIGTERM, TERM_handler);
+	signal(SIGHUP,  HUP_handler);
 	signal(SIGUSR1, USR1_handler);
+#ifdef HAVE_CACHE
 	signal(SIGUSR2, USR2_handler);
+#endif
 
 	/* Start listening for incoming connections
 	 */
@@ -1760,22 +1772,27 @@ int run_server(t_settings *settings) {
 				check_ban_list(config);
 				check_delayed_remove_timers(config);
 				remove_wrong_password_list(config);
+
 				/* FastCGI check
 				 */
 				check_load_balancer(config);
+
 				/* AccessLogfile check
 				 */
 				log_close(config->first_host, false);
+
 #ifdef HAVE_CACHE
 				/* Cache check
 				 */
 				check_cache();
 #endif
+
 #ifdef HAVE_COMMAND
 				/* CommandChannel check
 				 */
 				check_admin_list();
 #endif
+
 				switch (received_signal) {
 					case rs_NONE:
 						break;
@@ -1783,14 +1800,24 @@ int run_server(t_settings *settings) {
 						must_quit = true;
 						break;
 					case rs_UNBAN_CLIENTS:
-						unban_ip(0);
+						default_ipv4(&ip_addr);
+						unban_ip(&ip_addr);
+#ifdef HAVE_IPV6
+						default_ipv6(&ip_addr);
+						unban_ip(&ip_addr);
+#endif
 						received_signal = rs_NONE;
 						break;
-					case rs_TOUCH_LOGFILES:
+					case rs_UNLOCK_LOGFILES:
 						log_close(config->first_host, true);
-						touch_logfiles(config, NULL);
 						received_signal = rs_NONE;
 						break;
+#ifdef HAVE_CACHE
+					case rs_CLEAR_CACHE:
+						clear_cache();
+						received_signal = rs_NONE;
+						break;
+#endif
 				}
 				break;
 			default:
