@@ -92,27 +92,31 @@ static int password_callback(char *buffer, int size, int rwflag, void *data) {
 	return len;
 } 
 
-void ssl_init(char *buffer, int size) {
+int ssl_init(char *buffer, int size) {
 	int i;
 
 	SSL_library_init();
 	SSL_load_error_strings(); 
 
 	if (buffer != NULL) {
-		RAND_seed(buffer, size);
+		RAND_add(buffer, size, (double)size);
 	}
 
 	lockcount = CRYPTO_num_locks();
 	if (lockcount > 0) {
 		if ((locks = malloc(lockcount * sizeof(pthread_mutex_t))) != NULL) {
 			for (i = 0; i < lockcount; i++) {
-				pthread_mutex_init(&locks[i], NULL);
+				if (pthread_mutex_init(&locks[i], NULL) != 0) {
+					return -1;
+				}
 			}
 		}
     }
   
 	CRYPTO_set_locking_callback(locking_callback); 
 	CRYPTO_set_id_callback(id_callback);
+
+	return 0;
 }
 
 static int load_dh_params(SSL_CTX *context, char *file) {
@@ -274,14 +278,8 @@ int ssl_close(SSL **ssl) {
 	return result;
 }
 
-void ssl_cleanup(SSL_CTX *context) {
-	int i;
-
+void ssl_free(SSL_CTX *context) {
 	SSL_CTX_free(context);
-  
-	for (i = 0; i < lockcount; i++) {
-		pthread_mutex_destroy(&locks[i]);
-	}
 }
 
 int get_client_certificate(SSL *ssl_data, int size, char *subject, char *issuer) {
