@@ -5,13 +5,13 @@
 #include <openssl/bio.h>
 #include <openssl/ssl.h>
 
-void ssl_init(char *buffer, int size);
+int  ssl_init(char *buffer, int size);
 SSL_CTX *ssl_binding(char *keyfile, char *CA_cert, int verify_depth, char *dh_file, char *ciphers);
 int  ssl_accept(int sock, SSL **ssl, SSL_CTX *context);
 int  ssl_receive(SSL *ssl, char *buffer, unsigned int maxlength);
 int  ssl_send(SSL *ssl, char *buffer, unsigned int length);
 int  ssl_close(SSL **ssl);
-void ssl_cleanup(SSL_CTX *context);
+void ssl_free(SSL_CTX *context);
 int  get_client_certificate(SSL *ssl_data, int size, char *subject, char *issuer);
 
 #endif
