@@ -497,12 +497,12 @@ static int fix_windows_path(char *value, char *key) {
 
 void close_bindings(t_binding *binding) {
 	while (binding != NULL) {
+		close(binding->socket);
 #ifdef HAVE_SSL
 		if (binding->use_ssl) {
-			ssl_cleanup(binding->ssl_context);
+			ssl_free(binding->ssl_context);
 		}
 #endif
-		close(binding->socket);
 
 		binding = binding->next;
 	}
@@ -1940,7 +1940,6 @@ unsigned short get_throttlespeed(char *type, t_throttle *throttle) {
 }
 
 #ifdef HAVE_SSL
-#ifndef HAVE_DEV_URANDOM
 static int mod_memadd(char *src, int src_size, char *dest, int dest_size, int dest_ofs) {
 	int src_ofs = 0;
 
@@ -2007,4 +2006,3 @@ void fill_random_buffer(t_config *config, char *buffer, int size) {
 	}
 }
 #endif
-#endif
