@@ -42,12 +42,12 @@
 
 #define MAX_VOLATILE_SIZE      1 * MEGABYTE
 #define FILE_BUFFER_SIZE      32 * KILOBYTE
-#define MAX_CGI_HEADER         8 * KILOBYTE
-#define MAX_TRACE_HEADER       1 * KILOBYTE
+#define MAX_CGI_HEADER        16 * KILOBYTE
+#define CGI_BUFFER_SIZE       32 * KILOBYTE
+#define MAX_TRACE_HEADER       2 * KILOBYTE
 #define VALUE_SIZE            64
 #define WAIT_FOR_LOCK          3
 #define FILESIZE_BUFFER_SIZE  30
-#define CGI_BUFFER_SIZE       32 * KILOBYTE
 
 #define rs_QUIT       -1
 #define rs_DISCONNECT -2
@@ -87,26 +87,6 @@ int send_file(t_session *session) {
 #endif
 	session->mimetype = get_mimetype(session->extension, session->config->mimetype);
 
-	handle = -1;
-	if (session->accept_gzip) {
-		size = strlen(session->file_on_disk);
-		memcpy(session->file_on_disk + size, ".gz\0", 4);
-		if ((handle = open(session->file_on_disk, O_RDONLY)) != -1) {
-			session->encode_gzip = true;
-		} else {
-			*(session->file_on_disk + size) = '\0';
-		}
-	}
-	if (handle == -1) {
-		if ((handle = open(session->file_on_disk, O_RDONLY)) == -1) {
-			if (errno == EACCES) {
-				log_error(session, fb_filesystem);
-				return 403;
-			}
-			return 404;
-		}
-	}
-
 	/* Check the referer
 	 */
 	if (session->host->image_referer.size > 0) {
@@ -143,15 +123,42 @@ int send_file(t_session *session) {
 				free(session->file_on_disk);
 				if ((session->file_on_disk = (char*)malloc(strlen(session->host->imgref_replacement) + 4)) != NULL) { /* + 3 for ".gz" (gzip encoding) */
 					strcpy(session->file_on_disk, session->host->imgref_replacement);
+
+					if ((session->extension = strrchr(session->file_on_disk, '.')) != NULL) {
+						(session->extension)++;
+					} else {
+						session->extension = session->config->no_extension_as;
+					}
 					session->mimetype = get_mimetype(session->extension, session->config->mimetype);
 				} else {
-					close(handle);
 					return 500;
 				}
 			}
 		}
 	}
 	
+	/* Open the file for reading
+	 */
+	handle = -1;
+	if (session->accept_gzip) {
+		size = strlen(session->file_on_disk);
+		memcpy(session->file_on_disk + size, ".gz\0", 4);
+		if ((handle = open(session->file_on_disk, O_RDONLY)) != -1) {
+			session->encode_gzip = true;
+		} else {
+			*(session->file_on_disk + size) = '\0';
+		}
+	}
+	if (handle == -1) {
+		if ((handle = open(session->file_on_disk, O_RDONLY)) == -1) {
+			if (errno == EACCES) {
+				log_error(session, fb_filesystem);
+				return 403;
+			}
+			return 404;
+		}
+	}
+
 	/* Symlink check
 	 */
 	if (session->host->follow_symlinks == false) {
@@ -1390,7 +1397,7 @@ int handle_put_request(t_session *session) {
 		if (range_found) {
 			return 416;
 		}
-		if ((handle_write = open(session->file_on_disk, O_CREAT | O_WRONLY)) == -1) {
+		if ((handle_write = open(session->file_on_disk, O_CREAT|O_WRONLY, session->host->alter_fmode)) == -1) {
 			log_error(session, fb_filesystem);
 			return 403;
 		}
@@ -1430,11 +1437,7 @@ int handle_put_request(t_session *session) {
 		}
 	}
 
-	/* Change file mode
-	 */
-	if (fchmod(handle_write, session->host->alter_fmode) == -1) {
-		log_error(session, "can't change file mode (PUT)");
-	}
+	file_lock.l_type = F_UNLCK;
 
 	/* Handle upload range
 	 */
@@ -1487,30 +1490,34 @@ int handle_put_request(t_session *session) {
 		}
 
 		if ((file_size != NEW_FILE) && (range_found == false)) {
-			ftruncate(handle_write, session->uploaded_size);
+			if (ftruncate(handle_write, session->uploaded_size) == -1) {
+				result = 500;
+			}
 		}
 		
 		/* Write content
 		 */
-		if ((buffer = (char*)malloc(FILE_BUFFER_SIZE)) != NULL) {
-			while (total_written < session->uploaded_size) {
-				if ((bytes_read = read(handle_read, buffer, FILE_BUFFER_SIZE)) != -1) {
-					if (bytes_read == 0) {
-						break;
-					} else if (write_buffer(handle_write, buffer, bytes_read) != -1) {
-						total_written += bytes_read;
-					} else {
+		if (result != 500) {
+			if ((buffer = (char*)malloc(FILE_BUFFER_SIZE)) != NULL) {
+				while (total_written < session->uploaded_size) {
+					if ((bytes_read = read(handle_read, buffer, FILE_BUFFER_SIZE)) != -1) {
+						if (bytes_read == 0) {
+							break;
+						} else if (write_buffer(handle_write, buffer, bytes_read) != -1) {
+							total_written += bytes_read;
+						} else {
+							result = 500;
+							break;
+						}
+					} else if (errno != EINTR) {
 						result = 500;
 						break;
 					}
-				} else if (errno != EINTR) {
-					result = 500;
-					break;
 				}
+				free(buffer);
+			} else {
+				result = 500;
 			}
-			free(buffer);
-		} else {
-			result = 500;
 		}
 	}
 
