@@ -15,6 +15,7 @@
 #define XSLT_INDEX_LEN 12
 #define MAX_PARAMETERS 200 /* must be even */
 #define MAX_NAME_LEN 100
+#define MAX_HEADER_LEN 100
 
 extern char *hs_conlen;
 
@@ -24,9 +25,12 @@ static void add_parameter(const char **params, char *key, char *value, int *i) {
 	size_t key_len;
 	char *str;
 
-	if ((key == NULL) || (value == NULL)) {
+	if (key == NULL) {
 		return;
 	}
+	if (value == NULL) {
+		value = "";
+	}
 
 	if (*i + 2 > MAX_PARAMETERS) {
 		return;
@@ -61,9 +65,8 @@ static void add_parameter(const char **params, char *key, char *value, int *i) {
 static void add_headerfield(t_session *session, const char **params, char *header, char *key, int *i) {
 	char *value;
 
-	if ((value = get_headerfield(header, session->headerfields)) != NULL) {
-		add_parameter(params, key, value, i);
-	}
+	value = get_headerfield(header, session->headerfields);
+	add_parameter(params, key, value, i);
 }
 
 static void add_parameter_line(const char **params, char *line, char split, char *prefix, int *i) {
@@ -87,10 +90,10 @@ static void add_parameter_line(const char **params, char *line, char split, char
 }
 
 static const char **get_transform_parameters(t_session *session) {
+	char ip[MAX_IP_STR_LEN], value[20], variable[MAX_HEADER_LEN];
+	t_headerfield *headerfields;
 	const char **params;
 	int i = 0;
-	char ip[MAX_IP_STR_LEN];
-	char value[20];
 
 	if ((params = (const char**)malloc(sizeof(void*) * (MAX_PARAMETERS + 1))) != NULL) {
 		add_parameter(params, "REQUEST_METHOD", session->method, &i);
@@ -112,14 +115,12 @@ static const char **get_transform_parameters(t_session *session) {
 #endif
 			add_parameter(params, "HTTP_SCHEME", "http", &i);
 
-		if (session->remote_user != NULL) {
-			if (session->http_auth == basic) {
-				add_parameter(params, "AUTH_TYPE", "Basic", &i);
-			} else if (session->http_auth == digest) {
-				add_parameter(params, "AUTH_TYPE", "Digest", &i);
-			}
-			add_parameter(params, "REMOTE_USER", session->remote_user, &i);
+		if (session->http_auth == basic) {
+			add_parameter(params, "AUTH_TYPE", "Basic", &i);
+		} else if (session->http_auth == digest) {
+			add_parameter(params, "AUTH_TYPE", "Digest", &i);
 		}
+		add_parameter(params, "REMOTE_USER", session->remote_user, &i);
 
 		add_headerfield(session, params, "Accept:", "HTTP_ACCEPT", &i);
 		add_headerfield(session, params, "Accept-Charset:", "HTTP_ACCEPT_CHARSET", &i);
@@ -127,11 +128,24 @@ static const char **get_transform_parameters(t_session *session) {
 		add_headerfield(session, params, "Client-IP:", "HTTP_CLIENT_IP", &i);
 		add_headerfield(session, params, "From:", "HTTP_FROM", &i);
 		add_headerfield(session, params, "Host:", "HTTP_HOST", &i);
+		add_headerfield(session, params, "If-Modified-Since:", "HTTP_IF_MODIFIED_SINCE", &i);
+		add_headerfield(session, params, "If-Unmodified-Since:", "HTTP_IF_UNMODIFIED_SINCE", &i);
 		add_headerfield(session, params, "Range:", "HTTP_RANGE", &i);
 		add_headerfield(session, params, "Referer:", "HTTP_REFERER", &i);
 		add_headerfield(session, params, "User-Agent:", "HTTP_USER_AGENT", &i);
 		add_headerfield(session, params, "Via:", "HTTP_VIA", &i);
-		add_headerfield(session, params, "X-Forwarded-For:", "HTTP_X_FORWARDED_FOR", &i);
+
+		/* Convert X-* HTTP headers to HTTP_* variables
+		 */
+		headerfields = session->headerfields;
+		while (headerfields != NULL) {
+			if (strncmp(headerfields->data, "X-", 2) == 0) {
+				if (header_to_variable(headerfields->data, (char*)&variable, MAX_HEADER_LEN) != -1) {
+					add_parameter(params, variable, headerfields->data + headerfields->value_offset, &i);
+				}
+			}
+			headerfields = headerfields->next;
+		}
 
 		value[19] = '\0';
 		snprintf(value, 19, "%d", session->return_code);
@@ -141,9 +155,7 @@ static const char **get_transform_parameters(t_session *session) {
 			add_parameter(params, "HTTP_GENERATED_ERROR", value, &i);
 		}
 
-		if (session->vars != NULL) {
-			add_parameter(params, "QUERY_STRING", session->vars, &i);
-		}
+		add_parameter(params, "QUERY_STRING", session->vars, &i);
 		if (session->body != NULL) {
 			value[19] = '\0';
 			snprintf(value, 19, "%ld", session->content_length);
@@ -197,7 +209,9 @@ bool can_transform_with_xslt(t_session *session) {
 
 	/* Check extension
 	 */
-	if (strcmp(session->extension, "xml") != 0) {
+	if (session->extension == NULL) {
+		return false;
+	} else if (strcmp(session->extension, "xml") != 0) {
 		return false;
 	}
 
